"""Main entry point for mdllama CLI

This module handles command-line argument parsing and dispatches to the appropriate
CLI methods. All business logic is handled by the LLM_CLI class and its managers.
"""

import argparse
import os
from .version import __version__
from .release import check_github_release
from .cli import LLM_CLI

def get_version():
    """Get the version of mdllama."""
    return __version__

def main():
    """Main CLI entrypoint - handles argument parsing and dispatch to CLI methods."""
    parser = argparse.ArgumentParser(description="mdllama - A command-line interface for Ollama API and OpenAI-compatible endpoints")
    parser.add_argument('--version', action='version', version=f'%(prog)s {get_version()}')

    # Add provider and OpenAI-compatible endpoint option
    parser.add_argument('-p', '--provider', choices=['ollama', 'openai'], default=None, help='Provider to use: ollama or openai (default: ollama)')
    parser.add_argument('--openai-api-base', help='OpenAI-compatible API base URL (e.g. https://ai.hackclub.com)', default=None)

    # Create subparsers for different commands
    subparsers = parser.add_subparsers(dest="command", help="Command to run")

    # Check release command (moved here)
    subparsers.add_parser("check-release", help="Check for new stable and pre-releases of mdllama")

    # Setup command
    setup_parser = subparsers.add_parser("setup", help="Set up the CLI with Ollama or OpenAI-compatible configuration")
    setup_parser.add_argument("-p", "--provider", choices=["ollama", "openai"], default=None, help="Provider to set up: ollama or openai (default: ollama)")
    setup_parser.add_argument("--ollama-host", help="Ollama host URL")
    setup_parser.add_argument("--openai-api-base", help="OpenAI-compatible API base URL (e.g. https://ai.hackclub.com)")

    # List models command
    models_parser = subparsers.add_parser("models", help="List available models")
    models_parser.add_argument("-p", "--provider", choices=["ollama", "openai"], default=None, help="Provider to use: ollama or openai (default: ollama)")
    models_parser.add_argument("--openai-api-base", help=argparse.SUPPRESS)

    # Chat completion command
    chat_parser = subparsers.add_parser("chat", help="Generate a chat completion")
    chat_parser.add_argument("prompt", help="The prompt to send to the API", nargs="?")
    chat_parser.add_argument("--model", "-m", default="gemma3:1b", help="Model to use for completion")
    chat_parser.add_argument("--stream", "-s", action="store_true", help="Stream the response")
    chat_parser.add_argument("--system", help="System prompt to use")
    chat_parser.add_argument("--temperature", "-t", type=float, default=0.7, help="Temperature for sampling")
    chat_parser.add_argument("--max-tokens", type=int, help="Maximum number of tokens to generate")
    chat_parser.add_argument("--file", "-f", action="append", help="Path to file(s) to include as context (max 2MB per file)")
    chat_parser.add_argument("--context", "-c", action="store_true", help="Keep conversation context")
    chat_parser.add_argument("--save", action="store_true", help="Save conversation history")
    chat_parser.add_argument("--no-color", action="store_true", help="Disable colored output")
    chat_parser.add_argument("--render-markdown", "-r", action="store_true", help="Render markdown in the response")
    chat_parser.add_argument("-p", "--provider", choices=["ollama", "openai"], default=None, help="Provider to use: ollama or openai (default: ollama)")
    chat_parser.add_argument("--openai-api-base", help=argparse.SUPPRESS)
    chat_parser.add_argument("--prompt-file", help="Path to file containing the prompt")

    # Interactive chat command
    interactive_parser = subparsers.add_parser("run", help="Start an interactive chat session")
    interactive_parser.add_argument("--model", "-m", default=None, help="Model to use for completion (if not specified, will prompt to select)")
    interactive_parser.add_argument("--system", "-s", help="System prompt to use")
    interactive_parser.add_argument("--temperature", "-t", type=float, default=0.7, help="Temperature for sampling")
    interactive_parser.add_argument("--max-tokens", type=int, help="Maximum number of tokens to generate")
    interactive_parser.add_argument("--save", action="store_true", help="Save conversation history")
    interactive_parser.add_argument("--stream", action="store_true", help="Enable streaming responses")
    interactive_parser.add_argument("--no-color", action="store_true", help="Disable colored output")
    interactive_parser.add_argument("--render-markdown", "-r", action="store_true", help="Render markdown in the response")
    interactive_parser.add_argument("-p", "--provider", choices=["ollama", "openai"], default=None, help="Provider to use: ollama or openai (default: ollama)")
    interactive_parser.add_argument("--openai-api-base", help=argparse.SUPPRESS)

    # Context and history management
    subparsers.add_parser("clear-context", help="Clear the current conversation context")

    # Session management
    subparsers.add_parser("sessions", help="List available conversation sessions")
    load_parser = subparsers.add_parser("load-session", help="Load a conversation session")
    load_parser.add_argument("session_id", help="Session ID to load")

    # Ollama commands
    pull_parser = subparsers.add_parser("pull", help="Pull a model from Ollama registry")
    pull_parser.add_argument("model", help="Model name to pull")

    subparsers.add_parser("list", help="List all models in Ollama")
    subparsers.add_parser("ps", help="Show running model processes in Ollama")
    rm_parser = subparsers.add_parser("rm", help="Remove a model from Ollama")
    rm_parser.add_argument("model", help="Model name to remove")

    # Parse arguments
    args = parser.parse_args()

    # Determine if colors should be used
    use_colors = True
    if hasattr(args, 'no_color') and args.no_color:
        use_colors = False
    # Also check if NO_COLOR environment variable is set (common standard)
    if os.environ.get('NO_COLOR') is not None:
        use_colors = False

    # Initialize CLI
    render_markdown = False
    if hasattr(args, 'render_markdown') and args.render_markdown:
        render_markdown = True

    cli = LLM_CLI(use_colors=use_colors, render_markdown=render_markdown)

    # Handle commands
    if args.command == "check-release":
        check_github_release()
        return
    if args.command == "setup":
        provider = getattr(args, 'provider', None) or 'ollama'
        cli.setup(
            ollama_host=getattr(args, 'ollama_host', None),
            openai_api_base=getattr(args, 'openai_api_base', None),
            provider=provider
        )
    elif args.command == "models":
        provider = getattr(args, 'provider', None)
        openai_api_base = getattr(args, 'openai_api_base', None)
        cli.list_models(provider=provider, openai_api_base=openai_api_base)
    elif args.command == "chat":
        # Handle prompt from file or argument
        prompt = args.prompt
        if args.prompt_file:
            try:
                with open(args.prompt_file, 'r', encoding='utf-8') as f:
                    prompt = f.read().strip()
            except Exception as e:
                cli.output.print_error(f"Error reading prompt file: {e}")
                return
        
        if not prompt:
            cli.output.print_error("No prompt provided. Use --prompt-file or provide a prompt argument.")
            return
            
        provider = getattr(args, 'provider', None)
        openai_api_base = getattr(args, 'openai_api_base', None)
            
        cli.complete(
            prompt=prompt,
            model=args.model,
            stream=args.stream,
            system_prompt=args.system,
            temperature=args.temperature,
            max_tokens=args.max_tokens,
            file_paths=args.file,
            keep_context=args.context,
            save_history=args.save,
            provider=provider,
            openai_api_base=openai_api_base
        )
    elif args.command == "run":
        provider = getattr(args, 'provider', None) or 'ollama'
        
        # If no model specified, let user choose from available models
        model = args.model
        if not model:
            # Show model chooser at startup
            selected_model = cli.show_model_chooser(provider)
            if selected_model:
                model = selected_model
            else:
                # Exit if no valid model was selected
                cli.output.print_error("No model selected. Exiting.")
                return
        
        cli.interactive_chat(
            model=model,
            system_prompt=args.system,
            temperature=args.temperature,
            max_tokens=args.max_tokens,
            save_history=args.save,
            stream=args.stream,
            provider=provider
        )
    elif args.command == "clear-context":
        cli.clear_context()
    elif args.command == "sessions":
        cli.list_sessions()
    elif args.command == "load-session":
        cli.load_session(args.session_id)
    elif args.command == "pull":
        from .ollama_client import OllamaClient
        from .config import OLLAMA_DEFAULT_HOST
        
        ollama_client = OllamaClient(cli.config.get('ollama_host', OLLAMA_DEFAULT_HOST))
        if ollama_client.pull_model(args.model):
            cli.output.print_success(f"Model '{args.model}' pulled successfully.")
        else:
            cli.output.print_error(f"Failed to pull model '{args.model}'.")
    elif args.command == "list":
        cli.list_models()
    elif args.command == "ps":
        from .ollama_client import OllamaClient
        from .config import OLLAMA_DEFAULT_HOST
        
        ollama_client = OllamaClient(cli.config.get('ollama_host', OLLAMA_DEFAULT_HOST), use_colors=use_colors)
        try:
            models = ollama_client.list_running_models()
            ollama_client.format_running_models(models)
        except Exception as e:
            cli.output.print_error(f"Error listing running models: {e}")
    elif args.command == "rm":
        from .ollama_client import OllamaClient
        from .config import OLLAMA_DEFAULT_HOST
        
        ollama_client = OllamaClient(cli.config.get('ollama_host', OLLAMA_DEFAULT_HOST))
        if ollama_client.delete_model(args.model):
            cli.output.print_success(f"Model '{args.model}' removed successfully.")
        else:
            cli.output.print_error(f"Failed to remove model '{args.model}'.")
    else:
        # If no command is provided, print help
        parser.print_help()



if __name__ == "__main__":
    main()



"""Ollama API client for mdllama"""

import json
import requests
import sys
import datetime
from typing import List, Dict, Any, Optional, Generator, Callable
from .config import OLLAMA_DEFAULT_HOST

try:
    import ollama
    OLLAMA_AVAILABLE = True
except ImportError:
    OLLAMA_AVAILABLE = False

try:
    from rich.progress import Progress, BarColumn, TextColumn, TimeRemainingColumn, DownloadColumn, TransferSpeedColumn
    from rich.console import Console
    from rich.markdown import Markdown
    from rich.live import Live
    RICH_AVAILABLE = True
except ImportError:
    RICH_AVAILABLE = False

class OllamaClient:
    """Client for interacting with Ollama API."""
    
    def __init__(self, host: str = OLLAMA_DEFAULT_HOST, use_colors: bool = True, render_markdown: bool = False):
        self.host = host
        self.client = None
        self.use_colors = use_colors
        self.render_markdown = render_markdown
        self.console = Console() if RICH_AVAILABLE else None
        
    def setup_client(self) -> bool:
        """Initialize the Ollama client."""
        if not OLLAMA_AVAILABLE:
            return False
            
        try:
            # Check if Ollama is running by making a simple request
            test_response = requests.get(f"{self.host}/api/tags")
            if test_response.status_code != 200:
                return False
                
            # Initialize Ollama client with the host
            self.client = ollama.Client(host=self.host)
            return True
        except (requests.exceptions.ConnectionError, Exception):
            return False
            
    def is_available(self) -> bool:
        """Check if Ollama is available and running."""
        return OLLAMA_AVAILABLE and self.setup_client()
        
    def list_models(self) -> List[Dict[str, Any]]:
        """List available models from Ollama."""
        try:
            response = requests.get(f"{self.host}/api/tags")
            if response.status_code == 200:
                models_data = response.json()
                return models_data.get('models', [])
            else:
                raise Exception(f"HTTP {response.status_code}: {response.text}")
        except Exception as e:
            raise Exception(f"Error listing models: {e}")
            
    def pull_model(self, model_name: str, show_progress: bool = True) -> bool:
        """Pull a model from Ollama registry with progress display. Handles Ctrl+C gracefully."""
        try:
            url = f"{self.host}/api/pull"
            resp = requests.post(url, json={"name": model_name}, stream=True)
            if resp.status_code != 200:
                print(f"Error: {resp.status_code} {resp.text}")
                return False
            try:
                if show_progress and RICH_AVAILABLE:
                    with Progress(
                        TextColumn("[progress.description]{task.description}"),
                        BarColumn(),
                        DownloadColumn(),
                        TransferSpeedColumn(),
                        TimeRemainingColumn(),
                    ) as progress:
                        task = progress.add_task(f"Pulling {model_name}", total=None)
                        total = None
                        completed = 0
                        for line in resp.iter_lines():
                            if line:
                                data = json.loads(line)
                                if 'status' in data and data['status'] == 'success':
                                    progress.update(task, completed=total or completed)
                                    break
                                if 'total' in data:
                                    total = data['total']
                                    progress.update(task, total=total)
                                if 'completed' in data:
                                    completed = data['completed']
                                    progress.update(task, completed=completed)
                                if 'digest' in data:
                                    progress.update(task, description=f"Pulling {model_name} [{data['digest'][:12]}]")
                else:
                    # Simple fallback progress
                    total = None
                    completed = 0
                    for line in resp.iter_lines():
                        if line:
                            data = json.loads(line)
                            if 'status' in data and data['status'] == 'success':
                                print(f"\nPull complete: {model_name}")
                                break
                            if 'total' in data:
                                total = data['total']
                            if 'completed' in data:
                                completed = data['completed']
                            if total:
                                percent = (completed / total) * 100 if total else 0
                                sys.stdout.write(f"\rPulling {model_name}: {percent:.1f}% ({completed}/{total})")
                                sys.stdout.flush()
                    print()
            except KeyboardInterrupt:
                print(f"\nPull cancelled by user (Ctrl+C)")
                return False
            return True
        except Exception:
            return False
            
    def delete_model(self, model_name: str) -> bool:
        """Delete a model from Ollama."""
        try:
            url = f"{self.host}/api/delete"
            resp = requests.delete(url, json={"name": model_name})
            if resp.status_code == 200:
                return True
            else:
                return False
        except Exception:
            return False
            
    def list_running_models(self) -> List[Dict[str, Any]]:
        """List running model processes and return the data."""
        try:
            url = f"{self.host}/api/ps"
            resp = requests.get(url)
            if resp.status_code == 200:
                data = resp.json()
                models = data.get('models', [])
                return models
            else:
                raise Exception(f"HTTP {resp.status_code}: {resp.text}")
        except Exception as e:
            raise Exception(f"Error listing running models: {e}")
            
    def format_running_models(self, models: List[Dict[str, Any]]) -> None:
        """Format and print running models with detailed output."""
        if not models:
            print("No models are currently running.")
            return
        
        # Import Colors here to avoid circular imports
        from .colors import Colors
        
        # Print header
        if self.use_colors:
            print(f"{Colors.BOLD}NAME{Colors.RESET:<12} {Colors.BOLD}ID{Colors.RESET:<18} {Colors.BOLD}SIZE{Colors.RESET:<9} {Colors.BOLD}PROCESSOR{Colors.RESET:<7} {Colors.BOLD}UNTIL{Colors.RESET}")
        else:
            print("NAME         ID              SIZE      PROCESSOR    UNTIL")
        
        # Print each running model
        for model in models:
            name = model.get('name', 'Unknown')
            model_id = model.get('digest', '')[:12] if model.get('digest') else 'Unknown'
            size = model.get('size', 0)
            size_str = f"{size // (1024*1024)} MB" if size > 0 else "Unknown"
            
            # Format processor info based on size_vram and other details
            size_vram = model.get('size_vram', 0)
            details = model.get('details', {})
            
            if size_vram > 0:
                # Model is using GPU memory
                vram_mb = size_vram // (1024 * 1024)
                if vram_mb < 1024:
                    processor = f"GPU ({vram_mb}MB)"
                else:
                    vram_gb = vram_mb / 1024
                    processor = f"GPU ({vram_gb:.1f}GB)"
            else:
                # Model is using CPU - check if we have quantization info
                quant_level = details.get('quantization_level', '')
                if quant_level:
                    processor = f"CPU ({quant_level})"
                else:
                    processor = "100% CPU"
            
            # Format expiry time
            expires_at = model.get('expires_at', '')
            until = "Unknown"
            if expires_at:
                try:
                    # Parse the expiry time and calculate relative time
                    # Try to parse ISO format timestamp
                    if expires_at.endswith('Z'):
                        # Remove Z and treat as UTC
                        expires_at = expires_at[:-1] + '+00:00'
                    
                    # Parse the timestamp
                    if '+' in expires_at or expires_at.endswith('Z'):
                        # Has timezone info
                        expire_time = datetime.datetime.fromisoformat(expires_at.replace('Z', '+00:00'))
                        now = datetime.datetime.now(datetime.timezone.utc)
                    else:
                        # No timezone info, assume local time
                        expire_time = datetime.datetime.fromisoformat(expires_at)
                        now = datetime.datetime.now()
                    
                    diff = expire_time - now
                    if diff.total_seconds() > 0:
                        minutes = int(diff.total_seconds() / 60)
                        if minutes < 60:
                            until = f"{minutes} minutes from now"
                        else:
                            hours = minutes // 60
                            remaining_minutes = minutes % 60
                            if remaining_minutes > 0:
                                until = f"{hours}h {remaining_minutes}m from now"
                            else:
                                until = f"{hours} hours from now"
                    else:
                        until = "Expired"
                except Exception as e:
                    # Fallback if parsing fails
                    until = "Unknown"
            
            if self.use_colors:
                print(f"{Colors.BRIGHT_YELLOW}{name:<12}{Colors.RESET} {model_id:<16} {size_str:<9} {processor:<12} {until}")
            else:
                print(f"{name:<12} {model_id:<16} {size_str:<9} {processor:<12} {until}")
            
    def list_models_formatted(self) -> None:
        """List available models with formatted output."""
        try:
            url = f"{self.host}/api/tags"
            resp = requests.get(url)
            if resp.status_code == 200:
                data = resp.json()
                
                # Import Colors here to avoid circular imports
                from .colors import Colors
                
                for model in data.get('models', []):
                    name = model.get('name', 'Unknown')
                    size = model.get('size', 0)
                    if self.use_colors:
                        print(f"- {Colors.BRIGHT_YELLOW}{name}{Colors.RESET} ({size} bytes)")
                    else:
                        print(f"- {name} ({size} bytes)")
            else:
                raise Exception(f"HTTP {resp.status_code}: {resp.text}")
        except Exception as e:
            raise Exception(f"Error listing models: {e}")
            
    def chat(self, 
             messages: List[Dict[str, Any]], 
             model: str,
             stream: bool = False,
             temperature: float = 0.7,
             max_tokens: Optional[int] = None,
             keep_context: bool = False,
             current_context: Optional[List[Dict[str, Any]]] = None,
             process_links_callback: Optional[Callable[[str], str]] = None) -> Any:
        """Generate a chat completion using Ollama with full functionality from original."""
        
        # Use direct API endpoint for better control
        api_endpoint = f"{self.host.rstrip('/')}/api/chat"
        
        payload = {
            "model": model,
            "messages": messages,
            "stream": stream,
            "options": {}
        }
        
        if temperature != 0.7:
            payload["options"]["temperature"] = temperature
            
        if max_tokens:
            payload["options"]["num_predict"] = max_tokens
        
        if stream:
            return self._stream_response_generator(api_endpoint, payload)
        else:
            return self._non_stream_response_with_formatting(api_endpoint, payload, messages, keep_context, current_context, process_links_callback)
            
    def _stream_response_generator(self, api_endpoint: str, payload: Dict[str, Any]) -> Generator[Dict[str, Any], None, None]:
        """Generate streaming response chunks."""
        response = requests.post(api_endpoint, json=payload, stream=True)
        
        if response.status_code == 200:
            for line in response.iter_lines():
                if line:
                    try:
                        chunk = json.loads(line)
                        yield chunk
                    except json.JSONDecodeError:
                        continue
        else:
            raise Exception(f"Ollama API error: {response.status_code} - {response.text}")
            
    def _stream_response_with_formatting(self, api_endpoint: str, payload: Dict[str, Any], 
                                       messages: List[Dict[str, Any]], keep_context: bool,
                                       current_context: Optional[List[Dict[str, Any]]],
                                       process_links_callback: Optional[Callable[[str], str]]) -> Optional[str]:
        """Handle streaming response with formatting from original implementation."""
        full_response = ""
        response = requests.post(api_endpoint, json=payload, stream=True)
        
        if response.status_code == 200:
            buffer = ""
            
            # Import Colors here to avoid circular imports
            from .colors import Colors
            
            if self.render_markdown and RICH_AVAILABLE and self.console:
                # Use Rich for live markdown rendering
                accumulated_text = ""
                with Live(Markdown(""), console=self.console, refresh_per_second=10) as live_display:
                    for line in response.iter_lines():
                        if line:
                            try:
                                chunk = json.loads(line)
                                if 'message' in chunk and 'content' in chunk['message']:
                                    chunk_content = chunk['message']['content']
                                    if chunk_content:
                                        full_response += chunk_content
                                        accumulated_text += chunk_content
                                        try:
                                            live_display.update(Markdown(accumulated_text))
                                        except Exception:
                                            live_display.update(accumulated_text)
                            except json.JSONDecodeError:
                                continue
            else:
                # Standard streaming with link processing
                for line in response.iter_lines():
                    if line:
                        try:
                            chunk = json.loads(line)
                            if 'message' in chunk and 'content' in chunk['message']:
                                chunk_content = chunk['message']['content']
                                if chunk_content:
                                    buffer += chunk_content
                                    full_response += chunk_content
                                    
                                    # Process buffer when appropriate
                                    if len(buffer) > 100 or any(c in buffer for c in [' ', '\n', '.', ',', ')']):
                                        if self.use_colors:
                                            if process_links_callback:
                                                processed_buffer = process_links_callback(buffer)
                                            else:
                                                processed_buffer = buffer
                                            print(f"{Colors.GREEN}{processed_buffer}{Colors.RESET}", end="", flush=True)
                                        else:
                                            print(buffer, end="", flush=True)
                                        buffer = ""
                        except json.JSONDecodeError:
                            continue
                
                # Process any remaining text in the buffer
                if buffer:
                    if self.use_colors:
                        if process_links_callback:
                            processed_buffer = process_links_callback(buffer)
                        else:
                            processed_buffer = buffer
                        print(f"{Colors.GREEN}{processed_buffer}{Colors.RESET}", end="", flush=True)
                    else:
                        print(buffer, end="", flush=True)
                        
            print()  # Add final newline
            
            # Add to context if keeping context
            if keep_context and current_context is not None:
                user_message = messages[-1]  # Get the last user message
                current_context.append(user_message)
                current_context.append({"role": "assistant", "content": full_response})
                
            return full_response
        else:
            raise Exception(f"Ollama API error: {response.status_code} - {response.text}")
            
    def _non_stream_response_with_formatting(self, api_endpoint: str, payload: Dict[str, Any],
                                           messages: List[Dict[str, Any]], keep_context: bool,
                                           current_context: Optional[List[Dict[str, Any]]],
                                           process_links_callback: Optional[Callable[[str], str]]) -> Optional[str]:
        """Handle non-streaming response with formatting from original implementation."""
        response = requests.post(api_endpoint, json=payload)
        
        if response.status_code == 200:
            data = response.json()
            if 'message' in data and 'content' in data['message']:
                full_response = data['message']['content']
                
                # Import Colors here to avoid circular imports
                from .colors import Colors
                
                # For non-streaming responses with markdown
                if self.render_markdown and RICH_AVAILABLE and self.console:
                    self.console.print(Markdown(full_response))
                else:
                    # Process links in the full response
                    if process_links_callback:
                        processed_response = process_links_callback(full_response)
                    else:
                        processed_response = full_response
                        
                    if self.use_colors:
                        print(f"{Colors.GREEN}{processed_response}{Colors.RESET}")
                    else:
                        print(full_response)
                
                # Add to context if keeping context
                if keep_context and current_context is not None:
                    user_message = messages[-1]  # Get the last user message
                    current_context.append(user_message)
                    current_context.append({"role": "assistant", "content": full_response})
                    
                return full_response
            else:
                raise Exception("Invalid response format from Ollama API.")
        else:
            raise Exception(f"Ollama API error: {response.status_code} - {response.text}")
            
    def _stream_response(self, api_endpoint: str, payload: Dict[str, Any]) -> Generator[Dict[str, Any], None, None]:
        """Handle streaming response from Ollama (basic version for compatibility)."""
        response = requests.post(api_endpoint, json=payload, stream=True)
        
        if response.status_code == 200:
            for line in response.iter_lines():
                if line:
                    try:
                        chunk = json.loads(line)
                        yield chunk
                    except json.JSONDecodeError:
                        continue
        else:
            raise Exception(f"Ollama API error: {response.status_code} - {response.text}")
            
    def _non_stream_response(self, api_endpoint: str, payload: Dict[str, Any]) -> Dict[str, Any]:
        """Handle non-streaming response from Ollama (basic version for compatibility)."""
        response = requests.post(api_endpoint, json=payload)
        
        if response.status_code == 200:
            return response.json()
        else:
            raise Exception(f"Ollama API error: {response.status_code} - {response.text}")



"""Release utilities for mdllama: check for new stable and pre-releases on GitHub."""
import sys
import requests
from .version import __version__

def check_github_release():
    """Check for new stable and pre-releases on GitHub and alert the user."""
    import os
    repo = "QinCai-rui/mdllama"
    api_url = f"https://api.github.com/repos/{repo}/releases"
    headers = {}
    token = os.environ.get("GITHUB_TOKEN")
    if token:
        headers["Authorization"] = f"token {token}"
    try:
        resp = requests.get(api_url, headers=headers, timeout=10)
        if resp.status_code != 200:
            print(f"Failed to fetch releases from GitHub (status {resp.status_code})")
            if resp.status_code == 403:
                print("Your IP may be rate limited by GitHub. Set a GITHUB_TOKEN environment variable for higher limits.")
            sys.exit(1)
        releases = resp.json()
        if not releases:
            print("No releases found on GitHub.")
            sys.exit(0)
        current = __version__
        latest_stable = None
        latest_prerelease = None
        for rel in releases:
            if not rel.get("prerelease", False) and not latest_stable:
                latest_stable = rel
            if rel.get("prerelease", False) and not latest_prerelease:
                latest_prerelease = rel
            if latest_stable and latest_prerelease:
                break
        import re
        def ver(rel):
            return rel["tag_name"].lstrip("v") if rel else None
        def is_date_version(v):
            return bool(re.match(r"^\d{8}\.\d+$", v))
        def semver_tuple(v):
            return tuple(map(int, (v.split(".") + [0,0,0])[:3]))
        def print_release(rel, kind):
            if rel:
                print(f"    \033[1;33mLatest {kind} release:\033[0m \033[1;36m{rel['tag_name']}\033[0m - \033[4;34m{rel['html_url']}\033[0m\n")
        print(f"\033[1;37mCurrent version:\033[0m \033[1;32m{current}\033[0m\n")
        print(f"\033[1;36mLatest stable:\033[0m {ver(latest_stable) if latest_stable else 'None'}")
        print(f"\033[1;35mLatest prerelease:\033[0m {ver(latest_prerelease) if latest_prerelease else 'None'}\n")
        updated = False
        stable_ver = ver(latest_stable)
        pre_ver = ver(latest_prerelease)
        # Only alert for stable if it's truly newer
        if latest_stable and stable_ver != current:
            from datetime import datetime
            def get_published_at(rel):
                return rel.get("published_at")
            if is_date_version(current):
                if stable_ver and is_date_version(stable_ver):
                    # Both are date-based, compare as floats
                    try:
                        if float(stable_ver) > float(current):
                            print("\033[93mA new stable release is available!\033[0m")
                            print_release(latest_stable, "stable")
                            updated = True
                    except Exception:
                        pass
                else:
                    # Current is date-based, stable is semver: compare published_at
                    # Find the current release object
                    current_rel = None
                    for rel in releases:
                        if ver(rel) == current:
                            current_rel = rel
                            break
                    if current_rel and get_published_at(latest_stable) and get_published_at(current_rel):
                        try:
                            stable_time = datetime.fromisoformat(get_published_at(latest_stable).replace('Z', '+00:00'))
                            current_time = datetime.fromisoformat(get_published_at(current_rel).replace('Z', '+00:00'))
                            if stable_time > current_time:
                                print("\033[93mA new stable release is available!\033[0m")
                                print_release(latest_stable, "stable")
                                updated = True
                        except Exception:
                            pass
            else:
                # Current is semver, compare tuples
                if stable_ver and semver_tuple(stable_ver) > semver_tuple(current):
                    print("\033[93mA new stable release is available!\033[0m")
                    print_release(latest_stable, "stable")
                    updated = True
        # Pre-release alert: only if pre-release is newer than current
        if latest_prerelease:
            from datetime import datetime
            def get_published_at(rel):
                return rel.get("published_at")
            current_rel = None
            for rel in releases:
                if ver(rel) == current:
                    current_rel = rel
                    break
            # If both are date-based, or if current is semver and pre-release is date-based, compare publish dates
            if (is_date_version(current) and is_date_version(pre_ver)) or (not is_date_version(current) and is_date_version(pre_ver)):
                if current_rel and get_published_at(latest_prerelease) and get_published_at(current_rel):
                    try:
                        pre_time = datetime.fromisoformat(get_published_at(latest_prerelease).replace('Z', '+00:00'))
                        current_time = datetime.fromisoformat(get_published_at(current_rel).replace('Z', '+00:00'))
                        if pre_time > current_time:
                            print("\033[96mA new pre-release is available!\033[0m")
                            print_release(latest_prerelease, "pre-release")
                            updated = True
                    except Exception:
                        pass
            else:
                # For semver or other cases, just compare version string as before
                if pre_ver != current:
                    print("\033[96mA new pre-release is available!\033[0m")
                    print_release(latest_prerelease, "pre-release")
                    updated = True
        if not updated:
            print("\n\033[1;32mYou are using the latest version.\033[0m\n")
    except Exception as e:
        print(f"Error checking releases: {e}")
        sys.exit(1)

if __name__ == "__main__":
    check_github_release()



"""
mdllama - A command-line interface for Ollama API and OpenAI-compatible endpoints
"""

from .version import __version__
from .cli import LLM_CLI
from .main import main
from .session import SessionManager
from .model_manager import ModelManager
from .output import OutputFormatter
from .colors import Colors

__all__ = ['__version__', 'LLM_CLI', 'main', 'SessionManager', 'ModelManager', 'OutputFormatter', 'Colors']



"""Session management utilities for mdllama"""

from .history import HistoryManager
from .output import OutputFormatter
from .colors import Colors
from typing import Optional, Dict, Any, List

class SessionManager:
    def __init__(self, output: OutputFormatter, use_colors: bool = True):
        self.history = HistoryManager()
        self.output = output
        self.use_colors = use_colors
        self.current_context: List[Dict[str, Any]] = []

    def list_sessions(self):
        sessions = self.history.list_sessions()
        if not sessions:
            self.output.print_info("No session history found.")
            return
        self.output.print_info("Available sessions:")
        for session in sessions:
            session_id = session['id']
            if session['corrupted']:
                if self.use_colors:
                    print(f"- {Colors.YELLOW}{session_id}{Colors.RESET} {Colors.RED}(corrupted){Colors.RESET}")
                else:
                    print(f"- {session_id} (corrupted)")
            else:
                date = session['date']
                message_count = session['message_count']
                if self.use_colors:
                    print(f"- {Colors.YELLOW}{session_id}{Colors.RESET}: {Colors.WHITE}{date.strftime('%Y-%m-%d %H:%M:%S')}{Colors.RESET} ({message_count} messages)")
                else:
                    print(f"- {session_id}: {date.strftime('%Y-%m-%d %H:%M:%S')} ({message_count} messages)")

    def load_session(self, session_id: str) -> bool:
        try:
            self.current_context = self.history.load_history(session_id)
            self.history.current_session_id = session_id
            self.output.print_success(f"Loaded history from session {session_id}")
            return True
        except (FileNotFoundError, ValueError) as e:
            self.output.print_error(f"Error loading history: {e}")
            return False

    def clear_context(self):
        self.current_context = []
        self.output.print_success("Context cleared.")
    
    def prepare_messages(self, prompt: str, system_prompt: Optional[str] = None) -> List[Dict[str, Any]]:
        """Prepare messages for completion, including context."""
        messages = self.current_context.copy()

        # Add system prompt if provided and not already present
        if system_prompt and not any(m.get("role") == "system" for m in messages):
            messages.append({"role": "system", "content": system_prompt})
            
        # Add the text prompt
        messages.append({"role": "user", "content": prompt})
            
        return messages
    
    def update_context(self, user_message: Dict[str, Any], assistant_response: str):
        """Update context with new user message and assistant response."""
        self.current_context.append(user_message)
        self.current_context.append({"role": "assistant", "content": assistant_response})
    
    def save_history_if_requested(self, save_history: bool):
        """Save conversation history if requested."""
        if save_history:
            self.history.save_history(self.current_context)
            return self.history.current_session_id
        return None



from setuptools import setup, find_packages

setup(
    name="mdllama",
    version="4.0.0",
    description="A command-line interface for Ollama API",
    author="QinCai-rui",
    packages=find_packages(),
    install_requires=[
        "requests",
        "rich",
        "colorama",
        "ollama"
    ],
    python_requires=">=3.7",
    entry_points={
        "console_scripts": [
            "mdllama=mdllama.main:main"
        ]
    },
)






"""Input utilities for mdllama including readline support"""

import atexit
import os
from .config import CONFIG_DIR

# Handle readline import and setup
try:
    import readline
    READLINE_AVAILABLE = True
    
    HISTORY_FILE = str(CONFIG_DIR / "input_history")
    
    def setup_readline():
        """Setup readline for command history."""
        if not CONFIG_DIR.exists():
            CONFIG_DIR.mkdir(parents=True, exist_ok=True)
            
        if os.path.exists(HISTORY_FILE):
            try:
                readline.read_history_file(HISTORY_FILE)
            except (PermissionError, OSError):
                # Ignore permission errors when reading history file (common on macOS)
                pass
                
        def safe_write_history():
            try:
                readline.write_history_file(HISTORY_FILE)
            except (PermissionError, OSError):
                # Ignore permission errors when writing history file (common on macOS)
                pass
                
        atexit.register(safe_write_history)
        readline.set_history_length(1000)
        
    # Initialize readline on import
    setup_readline()
    
except ImportError:
    READLINE_AVAILABLE = False

def input_with_history(prompt: str) -> str:
    """Input function with readline history support if available."""
    if READLINE_AVAILABLE:
        try:
            return input(prompt)
        except KeyboardInterrupt:
            raise
    else:
        return input(prompt)

def read_multiline_input() -> str:
    """Read multiline input from the user, ending with triple quotes."""
    lines = []
    print('Enter your multiline input (end with """ on a new line):')
    
    while True:
        try:
            line = input()
            if line.strip() == '"""':
                break
            lines.append(line)
        except EOFError:
            break
            
    return "\n".join(lines)



"""Model management utilities for mdllama"""

from .ollama_client import OllamaClient
from .openai_client import OpenAIClient
from .output import OutputFormatter
from .config import OLLAMA_DEFAULT_HOST
from .colors import Colors
from typing import Optional, Dict, Any

class ModelManager:
    def __init__(self, output: OutputFormatter, config: Dict[str, Any]):
        self.output = output
        self.config = config

    def show_model_chooser(self, provider: str = "ollama", openai_api_base: Optional[str] = None) -> Optional[str]:
        """Show a numbered list of available models and allow user to choose."""
        models = []
        
        if provider == "openai":
            # Get OpenAI models
            api_base = openai_api_base or self.config.get('openai_api_base')
            if not api_base:
                self.output.print_error("OpenAI API base URL not configured.")
                return None
                
            openai_client = OpenAIClient(api_base, self.config)
            try:
                model_list, error = openai_client.get_models()
                if error:
                    self.output.print_error(f"Error listing OpenAI models: {error}")
                    return None
                models = model_list
            except Exception as e:
                self.output.print_error(f"Error listing OpenAI models: {e}")
                return None
                
        elif provider == "ollama":
            # Get Ollama models
            ollama_client = OllamaClient(self.config.get('ollama_host', OLLAMA_DEFAULT_HOST))
            if not ollama_client.is_available():
                self.output.print_error("Ollama is not available. Please make sure Ollama is running.")
                return None
                
            try:
                model_data = ollama_client.list_models()
                models = [model.get('name', 'Unknown') for model in model_data]
            except Exception as e:
                self.output.print_error(f"Error listing Ollama models: {e}")
                return None
        else:
            # Try both providers
            # Try Ollama first
            ollama_client = OllamaClient(self.config.get('ollama_host', OLLAMA_DEFAULT_HOST))
            if ollama_client.is_available():
                try:
                    model_data = ollama_client.list_models()
                    models = [model.get('name', 'Unknown') for model in model_data]
                except Exception as e:
                    pass
                    
            # Try OpenAI if no Ollama models found
            if not models and self.config.get('openai_api_base'):
                openai_client = OpenAIClient(self.config['openai_api_base'], self.config)
                try:
                    model_list, error = openai_client.get_models()
                    if not error:
                        models = model_list
                except Exception as e:
                    pass
        
        if not models:
            self.output.print_error("No models available.")
            return None
            
        # Display numbered list
        self.output.print_info("Available models:")
        for i, model in enumerate(models, 1):
            print(f"({i}) {Colors.BRIGHT_YELLOW}{model}{Colors.RESET}")
        
        print(f"\n{Colors.BRIGHT_BLACK}Tip: Enter 'q', 'quit', 'exit', or 'cancel' to abort model selection{Colors.RESET}")
                
        # Get user choice with retry logic
        max_attempts = 3
        for attempt in range(max_attempts):
            try:
                choice = input(f"\n{Colors.BRIGHT_CYAN}Enter model number (1-{len(models)}): {Colors.RESET}")
                    
                # Allow user to cancel
                if choice.lower() in ['q', 'quit', 'exit', 'cancel']:
                    self.output.print_info("Model selection cancelled.")
                    return None
                    
                choice_num = int(choice.strip())
                if 1 <= choice_num <= len(models):
                    selected_model = models[choice_num - 1]
                    self.output.print_success(f"Selected model: {selected_model}")
                    return selected_model
                else:
                    remaining_attempts = max_attempts - attempt - 1
                    if remaining_attempts > 0:
                        self.output.print_error(f"Invalid choice. Please enter a number between 1 and {len(models)}. {remaining_attempts} attempts remaining.")
                    else:
                        self.output.print_error("Invalid choice. Maximum attempts reached.")
                        return None
            except (ValueError, EOFError, KeyboardInterrupt):
                remaining_attempts = max_attempts - attempt - 1
                if remaining_attempts > 0:
                    self.output.print_error(f"Invalid input. Please enter a valid number. {remaining_attempts} attempts remaining.")
                else:
                    self.output.print_error("Invalid input. Maximum attempts reached.")
                    return None
        
        return None

    def list_models(self, provider: str = "ollama", openai_api_base: Optional[str] = None):
        """List available models."""
        # If provider is explicitly specified, use only that provider
        if provider == "openai":
            # Use OpenAI only
            api_base = openai_api_base or self.config.get('openai_api_base')
            if not api_base:
                self.output.print_error("OpenAI API base URL not configured. Use 'mdllama setup -p openai' to configure.")
                return
                
            openai_client = OpenAIClient(api_base, self.config)
            try:
                models, error = openai_client.get_models()
                if error:
                    self.output.print_error(f"Error listing OpenAI models: {error}")
                else:
                    self.output.print_info("Available OpenAI-compatible models:")
                    for model in models:
                        print(f"- {Colors.BRIGHT_YELLOW}{model}{Colors.RESET}")
                return
            except Exception as e:
                self.output.print_error(f"Error listing OpenAI models: {e}")
                return
                
        elif provider == "ollama":
            # Use Ollama only
            ollama_client = OllamaClient(self.config.get('ollama_host', OLLAMA_DEFAULT_HOST))
            if not ollama_client.is_available():
                self.output.print_error("Ollama is not available. Please make sure Ollama is running.")
                return
                
            try:
                models = ollama_client.list_models()
                self.output.print_info("Available Ollama models:")
                for model in models:
                    model_name = model.get('name', 'Unknown')
                    print(f"- {Colors.BRIGHT_YELLOW}{model_name}{Colors.RESET}")
                return
            except Exception as e:
                self.output.print_error(f"Error listing Ollama models: {e}")
                return
        
        # Default behavior - try both providers
        # Try Ollama first
        ollama_client = OllamaClient(self.config.get('ollama_host', OLLAMA_DEFAULT_HOST))
        if ollama_client.is_available():
            try:
                models = ollama_client.list_models()
                self.output.print_info("Available Ollama models:")
                for model in models:
                    model_name = model.get('name', 'Unknown')
                    print(f"- {Colors.BRIGHT_YELLOW}{model_name}{Colors.RESET}")
                return
            except Exception as e:
                self.output.print_error(f"Error listing Ollama models: {e}")
                
        # Try OpenAI if configured
        if self.config.get('openai_api_base'):
            openai_client = OpenAIClient(self.config['openai_api_base'], self.config)
            try:
                models, error = openai_client.get_models()
                if error:
                    self.output.print_error(f"Error listing OpenAI models: {error}")
                else:
                    self.output.print_info("Available OpenAI-compatible models:")
                    for model in models:
                        print(f"- {Colors.BRIGHT_YELLOW}{model}{Colors.RESET}")
                return
            except Exception as e:
                self.output.print_error(f"Error listing OpenAI models: {e}")
                
        self.output.print_error("No configured providers available.")
